/*--------------------------------------------------------------------------
 * Copyright (C) 1992,1993,1994,1995,1996,1997 Dallas Semiconductor Corporation. 
 * All rights Reserved. Printed in U.S.A.
 * This software is protected by copyright laws of
 * the United States and of foreign countries.
 * This material may also be protected by patent laws of the United States 
 * and of foreign countries.
 * This software is furnished under a license agreement and/or a
 * nondisclosure agreement and may only be used or copied in accordance
 * with the terms of those agreements.
 * The mere transfer of this software does not imply any licenses
 * of trade secrets, proprietary technology, copyrights, patents,
 * trademarks, maskwork rights, or any other form of intellectual
 * property whatsoever. Dallas Semiconductor retains all ownership rights.
 *--------------------------------------------------------------------------
 *
 *  GenUtils.C - General utility functions for iButton-TMEX applications
 *
 *  Compiler:  Microsoft Visual C 
 *  Version:   1.00Alpha
 *
 */

#include "ibtmex.h"   

/* Local Function Prototypes */
short ReadDefaultPort(void);
void Exit_Prog(char *, short);
void PrintHeader(char *);
short InterruptOperation(void);
short EnterNum(char *, short, ulong *);
short EnterStr(char *, short, char *);

/* external Function Prototypes */
extern void Release1WireNet(void);


/*----------------------------------------------------------------------
 * Read the defualt 1-Wire Net Port
 * return TRUE(1)  : Default port read
 *        FALSE(0) : Error reading default port
 */                                                          
short ReadDefaultPort(void)
{
   char ReturnString[256];
   HKEY GlobKey;
   DWORD dwType,size;

   /* read the default PortNum and PortType from the registry */
   if (RegOpenKeyEx(HKEY_LOCAL_MACHINE,
         "Software\\Dallas Semiconductor\\iButton TMEX\\3.00",0,
         KEY_READ,&GlobKey) == ERROR_SUCCESS)
   {
      /* attempt to read the PortNum */
      size = 255;
      PortNum = 1;
      if (RegQueryValueEx(GlobKey,"DefaultPortNum",NULL,&dwType,
          &ReturnString[0],&size) == ERROR_SUCCESS)
      {
         if ((dwType == REG_SZ) || (size > 0))
            PortNum = atoi(ReturnString);
      }
      /* attempt to read the PortType */
      size = 255;
      PortType = 1;
      if (RegQueryValueEx(GlobKey,"DefaultPortType",NULL,&dwType,
          &ReturnString[0],&size) == ERROR_SUCCESS)
      {
         if ((dwType == REG_SZ) || (size > 0))
            PortType = atoi(ReturnString);
      }

      // close the key
      RegCloseKey(GlobKey);
   }
   else
      return FALSE;

   /* check to see that indicated PortNum/PortType is  <= 15 */
   if ((PortNum > 15) || (PortType > 15))
      return FALSE;
   else
      return TRUE;
}

 
/*--------------------------------------------------------------------------
 *  Prints a message, beeps and exits the program.
 */
void Exit_Prog(char *msg, short errcd)
{
   /* print the message left justified */
   printf("\r%s",msg);
   
   /*  error beep if return code is not 0 */
   if (errcd != NORMAL_EXIT)
     MessageBeep(MB_ICONEXCLAMATION);
   
   /* close the port if setup done */
   if (SetupDone)
      TMClose(session_handle);

   /* close the session */
   Release1WireNet();

   /*  return error code passed */
   exit(errcd);
}

/*--------------------------------------------------------------------------
 *  Prints a message, and the current driver versions.
 */
void PrintHeader(char *msg)
{
   char tstr[80];       

   /* print a header with driver version info */  
   printf("\n%s\n"
         "Dallas Semiconductor Corporation\n"
         "Copyright (C) 1992-1999\n\n",msg);
   printf("Port number: %d    Port type: %d\n",PortNum,PortType);
   Get_Version(tstr);
   printf("Main driver version: %s\n",tstr);
   printf("Sub-driver version, Type%d:",PortType);
   if (TMGetTypeVersion(PortType,tstr) < 0)
      Exit_Prog("No Hardware Driver for this type found!",NO_DRIVERS);
   printf(" %s\n\n\n",tstr);
}

/*--------------------------------------------------------------------------
 * Detects an abort of current operation
 */
short InterruptOperation(void)
{
   return kbhit();
}

/*----------------------------------------------------------------------
 * Enter a decimal string and convert it to an unsigned long
 */                                                          
short EnterNum(char *msg, short numchars, ulong *value)
{
   short tmp,cnt;
   char ch;

   printf("%s",msg);

   *value = 0;

   cnt = 0;
   do
   {
      ch = getch();

      /* backspace */
      if (ch == 0x08)
      {
         printf(" %c",ch);
         if (cnt)
            cnt--;
      }
      /* escape */
      if (ch == 0x1B)
      {
         printf("  Aborted\n\n");
         return FALSE;
      }
      /* enter */
      if (ch == 0x0D)
      {
         printf("\n");
         return TRUE;
      }
      /* number */
      else if ((ch >= '0') && (ch <= '9'))
      {
         printf("%c",ch);
         tmp = ch - 0x30;
         *value *= 10; 
         *value += tmp;
         cnt++;
      }

   }
   while (cnt < numchars);

   printf("\n");

   return TRUE;
}


/*----------------------------------------------------------------------
 * Enter a hex string and convert it to an unsigned short
 */                                                          
short EnterStr(char *msg, short numchars, char *buf)
{
   short cnt;
   char ch;

   printf("%s",msg);

   cnt = 0;
   do
   {
      ch = getche();

      /* backspace */
      if (ch == 0x08)
      {
         printf(" %c",ch);
         if (cnt)
            cnt--;
      }
      /* escape */
      if (ch == 0x1B)
      {
         printf("  Aborted\n\n");
         return FALSE;
      }
      /* enter */
      if (ch == 0x0D)
      {
         buf[cnt] = 0;
         printf("\n");
         return TRUE;
      }
      /* ascii character */
      else if ((ch >= 0x20) && (ch < 0x7F))
         buf[cnt++] = ch;
   }
   while (cnt < numchars);

   buf[cnt] = 0;
   printf("\n");

   return TRUE;
}

