/*--------------------------------------------------------------------------
 * Copyright (C) 1992-1999 Dallas Semiconductor Corporation. 
 * All rights Reserved. Printed in U.S.A.
 * This software is protected by copyright laws of
 * the United States and of foreign countries.
 * This material may also be protected by patent laws of the United States 
 * and of foreign countries.
 * This software is furnished under a license agreement and/or a
 * nondisclosure agreement and may only be used or copied in accordance
 * with the terms of those agreements.
 * The mere transfer of this software does not imply any licenses
 * of trade secrets, proprietary technology, copyrights, patents,
 * trademarks, maskwork rights, or any other form of intellectual
 * property whatsoever. Dallas Semiconductor retains all ownership rights.
 *--------------------------------------------------------------------------
 *
 *  IBUtils.C - iButton utility functions  
 *
 *  Compiler:  Microsoft Visual C 
 *  Version:   1.00
 *
 */

#include "ibtmex.h"   

/* Local Function Prototypes */
short FindFirstFamily(short, long);
short Aquire1WireNet(void);
void Release1WireNet(void);
short WaitForiButton(uchar);
short iBReadFile(FileEntry *, uchar *, short *);
short iBWriteFile(FileEntry *, uchar *, short);
short VerifyCRCPacket(short, uchar *);
ushort docrc16(ushort);
uchar iBGetStartPage(FileEntry *);

/* external function prototypes */
extern short InterruptOperation(void);


/*----------------------------------------------------------------------
 * Aquire a session on the 1-Wire Net
 * return TRUE(1)  : session on 1-Wire Net aquired
 *        FALSE(0) : could not aquire a session on the 1-Wire Net
 */                                                          
short Aquire1WireNet(void)
{
   short printmsg=1;

   /* loop to get a session handle find an EPROM iButton */
   do
   {
      /* attempt to get a session */
      session_handle = TMExtendedStartSession(PortNum,PortType,NULL);
      if (session_handle > 0)  
      {
         sprintf(LastStatus,"Session on 1-Wire Net aquired");
         return TRUE;
      }
      else if (session_handle == 0)
      {
         sprintf(LastStatus,"Waiting to get access to 1-Wire Net");
         continue;
      }
      else if (session_handle < 0)   
      {
         sprintf(LastError,"No Hardware Driver for this type found!");
         sprintf(LastStatus,"%s",LastError);
         return FALSE;
      }
   }
   while (!InterruptOperation());

   sprintf(LastError,"Operation aborted\n");
   sprintf(LastStatus,"%s",LastError);

   return FALSE;   
}


/*----------------------------------------------------------------------
 * Give up the session on the 1-Wire Net
 */                                                          
void Release1WireNet(void)
{
   /* end the session if one is open */
   TMEndSession(session_handle);
   sprintf(LastStatus,"Release the 1-Wire Net");
}


/*----------------------------------------------------------------------
 * Get a session, do setup, and find a device
 * return TRUE(1)  : device found 
 *        FALSE(0) : device not found
 */                                                          
short WaitForiButton(uchar family)
{
   do
   {
      /* check if need to do setup  */
      if (!SetupDone)
      {
         if (TMSetup(session_handle) == 1)
            SetupDone = TRUE;
         else   
         {
            sprintf(LastError,"1-Wire Net setup failed!"); 
            sprintf(LastStatus,"%s",LastError);
            return FALSE;
         }
      }

      /* proceed with search */
      if (SetupDone)
      {  
         /* look for the first EPROM iButton */
         if (FindFirstFamily(family,session_handle))
            return TRUE;
         /* Temperature iButton not on MicroLan */
         else   
            sprintf(LastStatus,"Monetary iButton not found on 1-Wire Net!"); 
      }
   }
   while (!InterruptOperation());

   sprintf(LastError,"Operation aborted\n");
   sprintf(LastStatus,"%s",LastError);

   return FALSE;
}


/*----------------------------------------------------------------------
 * Read the specified file and place data and length in provided bufs.
 * Max file length 500 bytes.
 * return TRUE(1)  : File read successfully
 *        FALSE(0) : Error reading file
 */                                                          
short iBReadFile(FileEntry *flname, uchar *file_buf, short *file_len)
{
   short file_handle;

   /* attempt to open the file */
   file_handle = TMOpenFile(session_handle,state_buffer,flname);

   /* check if valid handle */
   if (file_handle >= 0)
   {
      /* proceed with read */
      *file_len = TMReadFile(session_handle,state_buffer,file_handle,
                          file_buf,500);
      TMCloseFile(session_handle,state_buffer,file_handle);
      /* check results */
      if (*file_len > 0)
      {
         sprintf(LastStatus,"File read successfully with length %d",*file_len);
         return TRUE;
      }
      else
      {
         sprintf(LastError,"Error reading file %d",*file_len); 
         sprintf(LastStatus,"%s",LastError);
         return FALSE;
      }   
   }
   else if (file_handle == XFILE_NOT_FOUND)
   {
      sprintf(LastError,"File to read not found"); 
      sprintf(LastStatus,"%s",LastError);
      return FALSE;
   }
   else
   {
      sprintf(LastError,"Error opening file %d",file_handle); 
      sprintf(LastStatus,"%s",LastError);
      return FALSE;
   }   
}

/*----------------------------------------------------------------------
 * Write the specified file and with data and length provided.
 * return TRUE(1)  : File written successfully
 *        FALSE(0) : Error reading file
 */                                                          
short iBWriteFile(FileEntry *flname, uchar *file_buf, short file_len)
{
   short file_handle,maxwrite,rslt;

   /* delete the file first */
   rslt = TMDeleteFile(session_handle,state_buffer,flname);
   /* ok if succes or file already deleted */
   if ((rslt == 1) || (rslt == XFILE_NOT_FOUND))
   {
      /* create the file */
      file_handle = TMCreateFile(session_handle,state_buffer,&maxwrite,flname);

      /* check if valid handle */
      if (file_handle >= 0)
      {
         /* proceed with write */
         rslt = TMWriteFile(session_handle,state_buffer,file_handle,
                             file_buf,file_len);
         TMCloseFile(session_handle,state_buffer,file_handle);
         /* check results */
         if (rslt > 0)
         {
            sprintf(LastStatus,"File written successfully");
            return TRUE;
         }
         else
         {
            sprintf(LastError,"Error writing file %d",rslt); 
            sprintf(LastStatus,"%s",LastError);
            return FALSE;
         }   
      }
      else
      {
         sprintf(LastError,"Error creating file %d",file_handle); 
         sprintf(LastStatus,"%s",LastError);
         return FALSE;
      }   
   }
   else
   {
      sprintf(LastError,"Error deleting file %d",rslt); 
      sprintf(LastStatus,"%s",LastError);
      return FALSE;
   }   
}


/*----------------------------------------------------------------------
 * Do a firstfile/nextfile to find the specified file name and return
 * the start page.  Only call if filename is known to exist and was
 * just read (lets us leave out some error conditions)
 */
uchar iBGetStartPage(FileEntry *target)
{
   FileEntry flname;
   short rslt;

   /* loop through all of the files in this directory */
   rslt = TMFirstFile(session_handle,state_buffer,&flname);
   while (rslt > 0)
   {
      /* compare to target */
      if ((flname.name[0] == target->name[0]) &&
          (flname.name[1] == target->name[1]) &&
          (flname.name[2] == target->name[2]) &&
          (flname.name[3] == target->name[3]) &&
          (flname.extension == target->extension))
         return flname.startpage;

      /* get the next file */
      rslt = TMNextFile(session_handle,state_buffer,&flname);
   }

   /* should not happen return 0 */
   return 0; 


}


/*----------------------------------------------------------------------
 * Find the first device with a particular family
 */
short FindFirstFamily(short family, long session_handle)
{
   short j,statuslen;
   
   /* set up to find the first device with family 'family */
   if (TMFamilySearchSetup(session_handle,&state_buffer,family) == 1)
   { 
      /* get first device in list with the specified family */
      if (TMNext(session_handle,state_buffer) == 1)
      {
         /* read the rom number */
         ROM[0] = 0;
         TMRom(session_handle,state_buffer,ROM);   
         
         /* check if correct type */
         if ((family & 0x7F) == (ROM[0] & 0x7F))
         {
            statuslen = sprintf(LastStatus,"Device found with registration number: ");  
            for (j = 7; j >= 0; j--)
               statuslen += sprintf(&LastStatus[statuslen],"%02X",ROM[j]);
            return TRUE;
         }        
      }
   }
     
   /* failed to find device of that type */
   return FALSE;                              
}


/*----------------------------------------------------------------------
 * Check provided buffer for a valid packet structure (max packet data
 * length 29 bytes)
 * return TRUE(1)  : packet valid
 *        FALSE(0) : packet not valid
 */                                                          
short VerifyCRCPacket(short seed, uchar *buf)
{
   short len,i;

   /* seed the crc */
   CRC16 = seed;

   /* get the length byte */
   len = buf[0];

   /* check for too large packet */
   if (len > 29)
      return FALSE;

   /* loop through the length byte, data, crc16 */
   for (i = 0; i < (len + 3); i++)
      docrc16(buf[i]);

   /* check for inverted magic number */
   return (CRC16 == 0xB001) ? TRUE : FALSE;
}

/*--------------------------------------------------------------------------
 * Calculate a new CRC16 from the input data shorteger.  Return the current
 * CRC16 and also update the global variable CRC16.
 */
static short oddparity[16] = { 0, 1, 1, 0, 1, 0, 0, 1, 1, 0, 0, 1, 0, 1, 1, 0 };

ushort docrc16(ushort data)
{
   data = (data ^ (CRC16 & 0xff)) & 0xff;
   CRC16 >>= 8;

   if (oddparity[data & 0xf] ^ oddparity[data >> 4])
     CRC16 ^= 0xc001;

   data <<= 6;
   CRC16   ^= data;
   data <<= 1;
   CRC16   ^= data;

   return CRC16;
}

