//
//   verf.h
//

#define DLLEXPORT __stdcall

#define TRUE  1
#define FALSE 0

#define INIT_H0 0x67452301
#define INIT_H1 0xEFCDAB89
#define INIT_H2 0x98BADCFE
#define INIT_H3 0x10325476
#define INIT_H4 0xC3D2E1F0

#define K0  0x5A827999
#define K20 0x6ED9EBA1
#define K40 0x8F1BBCDC
#define K60 0xCA62C1D6

#define BLOCK_SIZE_W 16
#define BLOCK_SIZE_B 64
#define HASH_LEN     20

#define MSG_LEN_MAX 1024

//
// Object type bytes from cibapi.h
//
#define CIB_OUTPUT_DATA      0   // Auto object types 
#define CIB_WORKING_REGISTER 1
#define CIB_ROM_DATA         2
#define CIB_RANDOM_FILL      3
#define CIB_MODULUS          32  // User objects 
#define CIB_EXPONENT         33
#define CIB_MONEY            34
#define CIB_COUNTER          35
#define CIB_SCRIPT           36
#define CIB_CLOCK_OFFSET     37
#define CIB_SALT             38
#define CIB_CONFIG_DATA      39
#define CIB_INPUT_DATA       40
#define CIB_DESTRUCTOR       41

typedef unsigned char uchar, *puchar;
typedef unsigned long ulong, *pulong;
typedef unsigned short ushort, *pushort;

typedef ulong MessageBlock[16];
typedef MessageBlock *pMessageBlock;
typedef ulong Hash[5];
typedef Hash *pHash;

void  Swap(puchar, puchar);
ulong S(ulong, uchar);
ulong Ft(int, ulong, ulong, ulong);
void  HashBlock(pulong, Hash);
void  PadMessage(puchar, ulong *, ulong);

#ifdef __cplusplus
extern "C" {
#endif

//
// Verf hashing functions
//
void DLLEXPORT LockHash(void);
void DLLEXPORT UnlockHash(void);
BOOL DLLEXPORT InitHash(Hash);
BOOL DLLEXPORT AddToBlob(puchar, ulong);
BOOL DLLEXPORT HashBlob(puchar, ulong, Hash);
BOOL DLLEXPORT HashMessage(puchar, ulong, Hash);

//
// Verf key/signature verification functions
//
void DLLEXPORT SetDallasPublicKey(ushort ExpLen, puchar pExp, 
                                  ushort ModLen, puchar pMod);
void DLLEXPORT GetDallasPublicKey(pushort pExpLen, puchar pExp,
                                   pushort pModLen, puchar pMod);
void DLLEXPORT SetCiBPublicKey(ushort ExpLen, puchar pExp, ushort ModLen, 
                               puchar pMod, puchar pSerialId);
void DLLEXPORT GetCiBPublicKey(pushort pExpLen, puchar pExp, pushort pModLen,
                                puchar pMod, puchar pSerialId);
BOOL DLLEXPORT VerifyCiBPublicKey(ushort CertLen, puchar pCert);
BOOL DLLEXPORT VerifySignature(ushort SigLen, puchar pSig, puchar pDigest);

#ifdef __cplusplus
}
#endif

